#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
############################################################################
# Istituto Superiore di Sanita'
# European Union Reference Laboratory (EU-RL) for Escherichia coli, including Verotoxigenic E. coli (VTEC)
# Developer: Arnold Knijn arnold.knijn@iss.it
############################################################################
"""

import argparse
import sys
import os
import HTML
import datetime

BASE_URL = 'https://galaxytrakr.org'
TOOL_DIR = os.path.dirname(os.path.abspath(__file__))

def insertFile(filename, report):
    with open(filename) as html_in:
        for line in html_in:
            report.write(line)

def insertFileAsTable(filename, report, hasheader=False, tabclass="table table-rep"):
    with open(filename) as table_in:
        table_data = [[str(col) for col in row.split('\t')] for row in table_in]
    insertTable(table_data, report, hasheader, tabclass)

def insertTable(table_data, report, hasheader=False, tabclass="table table-rep"):
    if hasheader:
        htmlcode = HTML.table(table_data[1:], attribs={'class':tabclass}, header_row=table_data[0])
    else:
        htmlcode = HTML.table(table_data, attribs={'class':tabclass})
    report.write(htmlcode)

def openFileAsTable(filename):
    with open(filename) as table_in:
        table_data = [[str(col) for col in row.split('\t')] for row in table_in]
    return table_data

def __main__():
    parser = argparse.ArgumentParser()
    parser.add_argument('--input_type', dest='input_type', help='type of input files (FASTQ or Contigs)')
    parser.add_argument('-1', '--input1', dest='input1', help='forward or single-end reads file in Sanger FASTQ format')
    parser.add_argument('--input1_ext', dest='input1_ext', help='extension of forward or single-end reads file in Sanger FASTQ format')
    parser.add_argument('--input1_name', dest='input1_name', help='name of forward or single-end reads file in Sanger FASTQ format')
    parser.add_argument('-2', '--input2', dest='input2', help='reverse reads file in Sanger FASTQ format')
    parser.add_argument('--input2_ext', dest='input2_ext', help='extension of reverse reads file in Sanger FASTQ format')
    parser.add_argument('--input2_name', dest='input2_name', help='name of reverse reads file in Sanger FASTQ format')
    parser.add_argument('--html1', dest='html1', help='html FASTQC file')
    parser.add_argument('--html1_id', dest='html1_id', help='html FASTQC file id')
    parser.add_argument('--html1_path', dest='html1_path', help='html FASTQC file path')
    parser.add_argument('--html2', dest='html2', help='html FASTQC file')
    parser.add_argument('--html2_id', dest='html2_id', help='html FASTQC file id')
    parser.add_argument('--html2_path', dest='html2_path', help='html FASTQC file path')
    parser.add_argument('--log', dest='logfile', help='log file')
    parser.add_argument('--spades_log', dest='spades_log', help='SPAdes log')
    parser.add_argument('--blastn_O', dest='blastn_O', help='Blastn for O')
    parser.add_argument('--blastn_H', dest='blastn_H', help='Blastn for H')
    parser.add_argument('--output', dest='output', help='output report html file')
    args = parser.parse_args()

    log = open(args.logfile, 'w')
    log.write("E coli Serotyper v1.2\n\nTool versions\n=============\n")
    if args.input2:
        # FASTQC
        os.system("python " + TOOL_DIR + "/scripts/rgFastQC.py -i " + args.input1 + " -d " + args.html1_path + " -o " + args.html1 + " -t text1 -f " + args.input1_ext + " -j " + args.input1_name + " -e " + "fastqc")
        os.system("rm -r " + args.html1_path)
        os.system("python " + TOOL_DIR + "/scripts/rgFastQC.py -i " + args.input2 + " -d " + args.html2_path + " -o " + args.html2 + " -t text2 -f " + args.input2_ext + " -j " + args.input2_name + " -e " + "fastqc")
        os.system("rm -r " + args.html2_path)
        log.write(os.popen('fastqc -v').read())
        # TRIMMING
        os.system("python " + TOOL_DIR + "/scripts/fastq_positional_quality_trimming.py -1 " + args.input1 + " --maxlt 300 --lt 17 --rt 0 --minqt 25 --avgqt 27.0 --minlf -1 --trimmed1 trimmed1 --log trimming_logfile -2 " + args.input2 + " --trimmed2 trimmed2 --trimmedunpaired trimmedunpaired")
        log.write("\nfastq_positional_quality_trimming v1.0\n")
        log.write("parameters: maxlt=300, lt=17, rt=0, minqt=25, avgqt=27.0, minlf=-1\n")
        # FILTERING
        os.system(TOOL_DIR + "/scripts/duk -m filteredO1.fq -k 23 " + TOOL_DIR + "/data/O_type.fsa trimmed1")
        os.system(TOOL_DIR + "/scripts/duk -m filteredH1.fq -k 23 " + TOOL_DIR + "/data/H_type.fsa trimmed1")
        os.system("cat filteredO1.fq > filteredOH1.fq")
        os.system("cat filteredH1.fq >> filteredOH1.fq")
        os.system(TOOL_DIR + "/scripts/duk -m filteredO2.fq -k 23 " + TOOL_DIR + "/data/O_type.fsa trimmed2")
        os.system(TOOL_DIR + "/scripts/duk -m filteredH2.fq -k 23 " + TOOL_DIR + "/data/H_type.fsa trimmed2")
        os.system("cat filteredO2.fq > filteredOH2.fq")
        os.system("cat filteredH2.fq >> filteredOH2.fq")
        os.system(TOOL_DIR + "/scripts/fastq_pair filteredOH1.fq filteredOH2.fq")
        os.system(TOOL_DIR + "/scripts/fastq_pair filteredOH1.fq.single.fq trimmed2")
        os.system(TOOL_DIR + "/scripts/fastq_pair filteredOH2.fq.single.fq trimmed1")
        os.system("cat filteredOH1.fq.paired.fq > filteredOH1_paired.fq")
        os.system("cat filteredOH1.fq.single.fq.paired.fq >> filteredOH1_paired.fq")
        os.system("cat trimmed1.paired.fq >> filteredOH1_paired.fq")
        os.system("cat filteredOH2.fq.paired.fq > filteredOH2_paired.fq")
        os.system("cat trimmed2.paired.fq >> filteredOH2_paired.fq")
        os.system("cat filteredOH2.fq.single.fq.paired.fq >> filteredOH2_paired.fq")
        log.write("\n\nFiltering\n=========\nduk v20110303\n")
        log.write("parameters: k=23\n")
        # ASSEMBLY: SPADES
        os.system("perl " + TOOL_DIR + "/scripts/spades.pl spades_contigs spades_contig_stats spades_scaffolds spades_scaffold_stats " + args.spades_log + " NODE spades.py --disable-gzip-output --careful -t \${GALAXY_SLOTS:-16} -k '21,33,55' --pe1-ff --pe1-1 fastq:filteredOH1_paired.fq --pe1-2 fastq:filteredOH2_paired.fq")
        log.write("\n\nAssembly\n========\n")
        log.write(os.popen("spades.py -v").read())
        log.write("parameters: careful, k=21,33,55, pe1-ff, pe1-1, pe1-2\n")
    else:
        if args.input_type == "fastq":
            # FASTQC
            os.system("python " + TOOL_DIR + "/scripts/rgFastQC.py -i " + args.input1 + " -d " + args.html1_path + " -o " + args.html1 + " -t text1 -f " + args.input1_ext + " -j " + args.input1_name + " -e " + "fastqc")
            os.system("rm -r " + args.html1_path)
            log.write(os.popen('fastqc -v').read())
            # TRIMMING
            os.system("python " + TOOL_DIR + "/scripts/fastq_positional_quality_trimming.py -1 " + args.input1 + " --maxlt 360 --lt 10 --rt 0 --minqt 25 --avgqt 27.0 --minlf 50 --trimmed1 trimmed1 --log trimming_logfile")
            log.write("fastq_positional_quality_trimming v1.0")
            log.write("parameters: maxlt=360, lt=10, rt=0, minqt=25, avgqt=27.0, minlf=50\n")
            # FILTERING
            os.system(TOOL_DIR + "/scripts/duk -m filteredO1.fq -k 23 " + TOOL_DIR + "/data/O_type.fsa trimmed1")
            os.system(TOOL_DIR + "/scripts/duk -m filteredH1.fq -k 23 " + TOOL_DIR + "/data/H_type.fsa trimmed1")
            os.system("cat filteredO1.fq > filteredOH1.fq")
            os.system("cat filteredH1.fq >> filteredOH1.fq")
            log.write("\n\nFiltering\n=========\nduk v20110303\n")
            log.write("parameters: k=23\n")
            # ASSEMBLY: SPADES
            os.system("perl " + TOOL_DIR + "/scripts/spades.pl spades_contigs spades_contig_stats spades_scaffolds spades_scaffold_stats " + args.spades_log + " NODE spades.py --disable-gzip-output --careful -t \${GALAXY_SLOTS:-16} --iontorrent -s fastq:filteredOH1.fq")
            log.write("\n\nAssembly\n========\n")
            log.write(os.popen("spades.py -v").read())
            log.write("parameters: careful, iontorrent\n")
        else:
            os.system("ln -s " + args.input1 + " spades_contigs")
    # NCBI BLAST+ blastn
    os.system("blastn -query 'spades_contigs' -db '" +  TOOL_DIR + "/data/O_type' -task blastn -evalue 0.001 -out '" + args.blastn_O + "' -outfmt '6 std sallseqid score nident positive gaps ppos qframe sframe qseq sseq qlen slen salltitles' -num_threads ${GALAXY_SLOTS:-8} -strand both -dust yes -max_target_seqs 10 -perc_identity 95.0")
    os.system("blastn -query 'spades_contigs' -db '" +  TOOL_DIR + "/data/H_type' -task blastn -evalue 0.001 -out '" + args.blastn_H + "' -outfmt '6 std sallseqid score nident positive gaps ppos qframe sframe qseq sseq qlen slen salltitles' -num_threads ${GALAXY_SLOTS:-8} -strand both -dust yes -max_target_seqs 10 -perc_identity 95.0 -ungapped")
    log.write("\n\nSeroTyper\n=========\n")
    log.write(os.popen("blastn -version").read())
    log.write("parameters: evalue=0.001, strand=both, dust=yes, max_target_seqs=10, perc_identity=95.0\n\n")
    log.write(os.popen("cat " +  TOOL_DIR + "/data/SeroTyping_db.txt").read())
    log.close()
    # SEROTYPER: FILTER, CUT AND CONCATENATE BLASTN OUTPUT
    os.system("echo 'sseqid\tpident\tlength\tpositive' > blastn_OH_fc")
    os.system("awk -F '\t' '$4>800 { print $2 FS $3 FS $4 FS $16 }' " + args.blastn_O + " | sort -nrk 2 -nrk 3 > blastn_O_fc")
    sero_typing_o = openFileAsTable("blastn_O_fc")
    os.system("cat blastn_O_fc >> blastn_OH_fc")
    os.system("awk -F '\t' '$4>800 { print $2 FS $3 FS $4 FS $16 }' " + args.blastn_H + " | sort -nrk 2 -nrk 3 > blastn_H_fc")
    sero_typing_h = openFileAsTable("blastn_H_fc")
    os.system("cat blastn_H_fc >> blastn_OH_fc")
    if os.stat('blastn_O_fc').st_size == 0 and os.stat('blastn_H_fc').st_size == 0:
        os.system("echo '-\t-\t-\t-' >> blastn_OH_fc")
    try:
        report = open(args.output, 'w')
        # write head html
        insertFile(TOOL_DIR + "/report_head.html", report)
        report.write("<td><h1><em>E coli</em> Serotyper</h1><h2>Report for %s</h2>%s</td>" % (args.input1_name, datetime.datetime.utcnow().strftime("%Y-%m-%d %H:%M UTC"))) 
        insertFile(TOOL_DIR + "/report_head2.html", report)
        # write results
        report.write("<h3>Summary</h3>\n")
        if len(sero_typing_o) == 0:
            report.write("O?")
        else:
            report.write("%s" % sero_typing_o[0][0][sero_typing_o[0][0].rfind("O"):])
        if len(sero_typing_h) == 0:
            report.write(":H?")
        else:
            report.write(":%s" % sero_typing_h[0][0][sero_typing_h[0][0].rfind("H"):])
        if args.input_type == "fastq":
            report.write("<h3>Raw data quality check</h3>\n")
            if args.input2:
                report.write("<p>FASTQC result forward: <a href='%s/datasets/%s/display/?preview=True'>Webpage</a></p>\n" % (BASE_URL, args.html1_id))
                report.write("<p>FASTQC result reverse: <a href='%s/datasets/%s/display/?preview=True'>Webpage</a></p>\n" % (BASE_URL, args.html2_id))
            else:
                report.write("<p>FASTQC result: <a href='%s/datasets/%s/display/?preview=True'>Webpage</a></p>\n" % (BASE_URL, args.html1_id))
        report.write("<br/><hr/><h3>Serotyping</h3>\n")
        insertFileAsTable("blastn_OH_fc", report, True)
        # write tail html
        insertFile(TOOL_DIR + "/report_tail.html", report)
    finally:
        report.close()

if __name__ == "__main__":
    __main__()


